@@ -20,6 +20,7 @@ from os.path import expandvars
 from os.path import isdir
 from os.path import isfile
 from os.path import join
+from sys import stdout
 from tempfile import gettempdir
 
 from . import __version__
@@ -54,7 +55,7 @@ def yml_read(yml_path):
         return f_yml.read()
 
 
-def main():
+def main(return_result=False):
     parser = argparse.ArgumentParser()
     parser.add_argument(
         "git_repo_url",
@@ -148,6 +149,21 @@ def main():
     parser.add_argument(
         '-v', '--version', action='version', version='%(prog)s ' + __version__
     )
+    parser.add_argument(
+        '--runs-at-the-end-script', dest='runs_at_the_end_script',
+        nargs='*', default="",
+        help='Extra commands to run after "script" file. '
+        'Note: You can use \\$IMAGE escaped environment variable.',
+    )
+    parser.add_argument(
+        '--build-env-args', dest='build_env_args',
+        nargs='*', action='append', default=[],
+        help='Args used as environment variables '
+        'More info about: https://vsupalov.com/docker-build-time-env-values\n'
+        'E.g. --build-env-args ENVAR1\n'
+        'It generates the following line for Dockerfile:\n'
+        'ARG ENVVAR1\nENV ENVVAR1=$ENVVAR1',
+    )
 
     args = parser.parse_args()
     revision = args.git_revision
@@ -165,6 +181,9 @@ def main():
     run_extra_cmds = '\n'.join(args.run_extra_cmds)
     no_clone = args.no_clone
     rcfiles_args = args.add_rcfile and args.add_rcfile.split(',')
+    runs_at_the_end_script = args.runs_at_the_end_script or None
+    build_env_args = [
+        build_env_args[0] for build_env_args in args.build_env_args]
     rcfiles = [
         (expanduser(rc_file), os.path.join('$HOME', os.path.basename(rc_file)))
         for rc_file in rcfiles_args]
@@ -205,6 +224,8 @@ def main():
         image=default_docker_image,
         os_kwargs=os_kwargs,
         copy_paths=[(expanduser("~/.ssh"), "$HOME/.ssh")] + rcfiles,
+        runs_at_the_end_script=runs_at_the_end_script,
+        build_env_args=build_env_args,
     )
     t2d.build_extra_params = {
         'extra_params': build_extra_args,
@@ -214,4 +235,11 @@ def main():
         'extra_params': run_extra_args,
         'extra_cmds': run_extra_cmds,
     }
-    return t2d.compute_dockerfile(skip_after_success=exclude_after_success)
+    fname_scripts = t2d.compute_dockerfile(skip_after_success=exclude_after_success)
+    if fname_scripts:
+        fname_list = '- ' + '\n- '.join(fname_scripts)
+        stdout.write('\nGenerated scripts:\n%s\n' % fname_list)
+    else:
+        stdout.write('\nNo scripts were generated.')
+    if return_result:
+        return fname_scripts
