@@ -8,7 +8,11 @@ import stat
 from tempfile import gettempdir
 
 import jinja2
-import yaml
+
+try:
+    from yaml import full_load as yaml_load
+except ImportError:
+    from yaml import load as yaml_load
 
 RE_ENV_STR = r"(?P<var>[\w]*)[ ]*[\=][ ]*[\"\']{0,1}" + \
              r"(?P<value>[\w\.\-\_/\$\{\}\:,\(\)\#\* ]*)[\"\']{0,1}"
@@ -60,12 +64,17 @@ class Travis2Docker(object):
 
     def __init__(self, yml_buffer, image=None, work_path=None, dockerfile=None,
                  templates_path=None, os_kwargs=None, copy_paths=None,
+                 runs_at_the_end_script=None, build_env_args=None,
                  ):
         self._python_versions = []
         self.curr_work_path = None
         self.curr_exports = []
         self.build_extra_params = {}
         self.run_extra_params = {}
+        self.build_env_args = build_env_args
+        self.runs_at_the_end_script = (
+            ["sleep 2"] if runs_at_the_end_script is None
+            else runs_at_the_end_script)
         if image is None:
             image = 'vauxoo/odoo-80-image-shippable-auto'
         if os_kwargs is None:
@@ -93,7 +102,7 @@ class Travis2Docker(object):
         self._sections['install'] = 'run'
         self._sections['script'] = 'entrypoint'
         self._sections['after_success'] = 'entrypoint'
-        self.yml = yaml.load(yml_buffer)
+        self.yml = yaml_load(yml_buffer)
         if work_path is None:
             base_name = os.path.splitext(os.path.basename(__file__))[0]
             self.work_path = os.path.join(gettempdir(), base_name)
@@ -107,19 +116,21 @@ class Travis2Docker(object):
         self.ubuntu_json = json.load(
             open(os.path.join(travis_ci_apt_src, "ubuntu.json")))
 
-    def _compute(self, section):
+    def _compute(self, section, yml=None):
+        if yml is None:
+            yml = self.yml
         section_type = self._sections.get(section)
         if not section_type:
             return None
-        section_data = self.yml.get(section, "")
+        section_data = yml.get(section, "")
         if section != "env" and not section_data:
             return None
         if not isinstance(section_data, (list, dict, tuple)):
             section_data = [section_data]
         job_method = getattr(self, '_compute_' + section_type)
-        return job_method(section_data, section)
+        return job_method(section_data, section, yml)
 
-    def _compute_env(self, data, _):
+    def _compute_env(self, data, _, yml):
         if isinstance(data, list):
             # old version without matrix
             data = {'matrix': data}
@@ -130,22 +141,23 @@ class Travis2Docker(object):
                 continue
             env_globals += " " + env_global
         env_globals = env_globals.strip()
-        psql_version = (self.yml.get('addons') or {}).get('postgresql')
+        psql_version = (yml.get('addons') or {}).get('postgresql')
         if psql_version:
             env_globals += ' PSQL_VERSION="%s"' % psql_version
-        for env_matrix in data.get('matrix', []):
+
+        for env_matrix in data.get('matrix', ['']):
             yield (env_globals + " " + env_matrix).strip()
 
-    def _compute_run(self, data, section):
+    def _compute_run(self, data, section, _):
         args = self._make_script(data, section, add_run=True, prefix='files')
         return args
 
-    def _compute_entrypoint(self, data, section):
+    def _compute_entrypoint(self, data, section, _):
         args = self._make_script(data, section, add_entrypoint=True,
                                  prefix='files')
         return args
 
-    def _compute_addons(self, data, section):
+    def _compute_addons(self, data, section, _):
         if 'apt' not in data:
             return
         sources = []
@@ -182,7 +194,8 @@ class Travis2Docker(object):
                     for _, _, var, value in self.re_export.findall(line)])
                 f_section.write('\n' + line)
             if section == 'script':
-                f_section.write('\nsleep 2\n')
+                for run_at_the_end_script in self.runs_at_the_end_script:
+                    f_section.write('\n%s' % run_at_the_end_script)
         src = "./" + os.path.relpath(file_path, self.curr_work_path)
         dest = "/" + section
         args = {
@@ -228,7 +241,7 @@ class Travis2Docker(object):
     def _python_version_env(self):
         versions = self.yml.pop('python', {})
         if not versions:
-            return False
+            return ['3.5']  # 3.5 by default
         if not isinstance(versions, list):
             versions = [versions]
         # TODO: Use full version if in the default base image are installed
@@ -248,71 +261,90 @@ class Travis2Docker(object):
         work_paths = []
         self._transform_yml_matrix2env()
         self._python_version_env()
-        for version in self._python_versions:
-            for count, env in enumerate(self._compute('env') or [], 1):
-                self.reset()
-                self.curr_work_path = os.path.join(self.work_path,
-                                                   version.replace('.', '_'),
-                                                   str(count))
-                curr_dockerfile = \
-                    os.path.join(self.curr_work_path, self.dockerfile)
-                entryp_path = os.path.join(self.curr_work_path, "files",
-                                           "entrypoint.sh")
-                self.mkdir_p(os.path.dirname(entryp_path))
-                entryp_relpath = os.path.relpath(entryp_path,
-                                                 self.curr_work_path)
-                rvm_env_path = os.path.join(self.curr_work_path, "files",
-                                            "rvm_env.sh")
-                rvm_env_relpath = os.path.relpath(rvm_env_path,
-                                                  self.curr_work_path)
-                copies = []
-                for copy_path, dest in self.copy_paths:
-                    copies.append((self.copy_path(copy_path), dest))
-                kwargs = {'runs': [], 'copies': copies, 'entrypoints': [],
-                          'entrypoint_path': entryp_relpath,
-                          'python_version': version,
-                          'image': self.image, 'env': env, 'packages': [],
-                          'sources': [], 'rvm_env_path': rvm_env_relpath}
-                with open(curr_dockerfile, "w") as f_dockerfile, \
-                        open(entryp_path, "w") as f_entrypoint, \
-                        open(rvm_env_path, "w") as f_rvm:
-                    for section, _ in self._sections.items():
-                        if section == 'env':
-                            continue
-                        if skip_after_success and section == 'after_success':
-                            continue
-                        result = self._compute(section)
-                        if not result:
-                            continue
-                        keys_to_extend = ['copies', 'runs', 'entrypoints',
-                                          'packages', 'sources'] \
-                            if isinstance(result, dict) else []
-                        for key_to_extend in keys_to_extend:
-                            if key_to_extend in result:
-                                kwargs[key_to_extend].extend(
-                                    result[key_to_extend])
-                    kwargs.update(self.os_kwargs)
-                    dockerfile_content = \
-                        self.dockerfile_template.render(kwargs).strip('\n ')
+        jobs_stages = self.yml.pop('jobs', {}).get('include', {})
+        for global_version in self._python_versions:
+            for count, global_env in enumerate(self._compute('env'), 1):
+                for job_count, job_stage in enumerate(jobs_stages or [{}], 1):
+                    job_env = self._compute('env', job_stage) or ""
+                    if job_env is not None:
+                        job_env = next(job_env)
+                    env = '%s %s' % (global_env, job_env)
+                    env = env.strip()
+                    version = global_version
                     try:
-                        f_dockerfile.write(dockerfile_content.encode('utf-8'))
-                    except TypeError:
-                        f_dockerfile.write(dockerfile_content)
-                    entrypoint_content = \
-                        self.entrypoint_template.render(kwargs).strip('\n ')
-                    try:
-                        f_entrypoint.write(entrypoint_content.encode('utf-8'))
-                    except TypeError:
-                        f_entrypoint.write(entrypoint_content)
-                    rvm_env_content = self.jinja_env.get_template(
-                        'rvm_env.sh').render(kwargs).strip('\n ')
-                    try:
-                        f_rvm.write(rvm_env_content.encode('UTF-8'))
-                    except TypeError:
-                        f_rvm.write(rvm_env_content)
-                self.compute_build_scripts(count, version)
-                self.chmod_execution(entryp_path)
-                work_paths.append(self.curr_work_path)
+                        version = job_stage['python']
+                    except KeyError:
+                        pass
+                    version = "%s" % version
+
+                    self.reset()
+                    self.curr_work_path = os.path.join(self.work_path,
+                                                       version.replace('.', '_'),
+                                                       "env_%d_job_%d" % (count, job_count))
+                    curr_dockerfile = \
+                        os.path.join(self.curr_work_path, self.dockerfile)
+                    entryp_path = os.path.join(self.curr_work_path, "files",
+                                               "entrypoint.sh")
+                    self.mkdir_p(os.path.dirname(entryp_path))
+                    entryp_relpath = os.path.relpath(entryp_path,
+                                                     self.curr_work_path)
+                    rvm_env_path = os.path.join(self.curr_work_path, "files",
+                                                "rvm_env.sh")
+                    rvm_env_relpath = os.path.relpath(rvm_env_path,
+                                                      self.curr_work_path)
+                    copies = []
+                    for copy_path, dest in self.copy_paths:
+                        copies.append((self.copy_path(copy_path), dest))
+                    kwargs = {'runs': [], 'copies': copies, 'entrypoints': [],
+                              'entrypoint_path': entryp_relpath,
+                              'python_version': version,
+                              'image': self.image, 'env': env, 'packages': [],
+                              'sources': [], 'rvm_env_path': rvm_env_relpath,
+                              'build_env_args': self.build_env_args,
+                              }
+                    with open(curr_dockerfile, "w") as f_dockerfile, \
+                            open(entryp_path, "w") as f_entrypoint, \
+                            open(rvm_env_path, "w") as f_rvm:
+                        for section, _ in self._sections.items():
+                            if section == 'env':
+                                continue
+                            if skip_after_success and section == 'after_success':
+                                continue
+                            # job section replace global one
+                            result = self._compute(section, job_stage)
+                            if not result:
+                                result = self._compute(section)
+                            if not result:
+                                continue
+                            keys_to_extend = ['copies', 'runs', 'entrypoints',
+                                              'packages', 'sources'] \
+                                if isinstance(result, dict) else []
+                            for key_to_extend in keys_to_extend:
+                                if key_to_extend in result:
+                                    kwargs[key_to_extend].extend(
+                                        result[key_to_extend])
+                        kwargs.update(self.os_kwargs)
+                        dockerfile_content = \
+                            self.dockerfile_template.render(kwargs).strip('\n ')
+                        try:
+                            f_dockerfile.write(dockerfile_content.encode('utf-8'))
+                        except TypeError:
+                            f_dockerfile.write(dockerfile_content)
+                        entrypoint_content = \
+                            self.entrypoint_template.render(kwargs).strip('\n ')
+                        try:
+                            f_entrypoint.write(entrypoint_content.encode('utf-8'))
+                        except TypeError:
+                            f_entrypoint.write(entrypoint_content)
+                        rvm_env_content = self.jinja_env.get_template(
+                            'rvm_env.sh').render(kwargs).strip('\n ')
+                        try:
+                            f_rvm.write(rvm_env_content.encode('UTF-8'))
+                        except TypeError:
+                            f_rvm.write(rvm_env_content)
+                    self.compute_build_scripts(count, version)
+                    self.chmod_execution(entryp_path)
+                    work_paths.append(self.curr_work_path)
         self.reset()
         return work_paths
 
